@@ -53,8 +53,7 @@ static int deviceid;
 
 static const char* white_list[] = {
     "LVDS",
-    "LVDS1",
-    "eDP1"
+    "eDP"
 };
 
 void search_touchscreen_device(Display *display)
@@ -107,7 +106,7 @@ void get_display_info(Display *display)
 
             for (l = 0; l < size; l++)
             {
-                if (strcmp(white_list[l], output->name) == 0)
+                if (strstr(output->name, white_list[l]))
                 {
                     for (j = 0; j < output->nmode; j++)
                     {
@@ -439,13 +438,27 @@ void scaling_full_aspect_mode(Display *display)
     apply_matrix(display, deviceid, m);
 }
 
-void routine(Display *display)
+void scaling_none_mode(Display *display)
 {
-    XCloseDisplay(display);
-    display = XOpenDisplay(NULL);
+    float m[9] = {
+        1.0f, 0    , 0,
+        0   , 1.0f , 0,
+        0   , 0    , 1.0f
+    };
 
-    search_touchscreen_device(display);
-    get_display_info(display);
+    rotate_reflect(m);
+
+    apply_matrix(display, deviceid, m);
+}
+
+void routine(Display **display)
+{
+    XCloseDisplay(*display);
+    usleep(100000); /* It needs to wait for a while before X resources are ready. */
+    *display = XOpenDisplay(getenv("DISPLAY"));
+
+    search_touchscreen_device(*display);
+    get_display_info(*display);
 
     if (touch_screen) {
         printf("Touchscreen: '%s'\n", touch_screen);
@@ -482,28 +495,29 @@ void routine(Display *display)
                 0   , 1.0f , 0,
                 0   , 0    , 1.0f
             };
-            apply_matrix(display, deviceid, m);
+            apply_matrix(*display, deviceid, m);
         } else {
             switch (scaling_mode) {
                 case ScalingMode_None:
                     printf(" 'None'");
+                    scaling_none_mode(*display);
                     break;
                 case ScalingMode_Full:
-                    scaling_full_mode(display);
+                    scaling_full_mode(*display);
                     break;
                 case ScalingMode_Center:
-                    scaling_center_mode(display);
+                    scaling_center_mode(*display);
                     break;
                 case ScalingMode_Full_aspect:
-                    scaling_full_aspect_mode(display);
+                    scaling_full_aspect_mode(*display);
                     break;
             }
         }
     }
 
-    XRRSelectInput(display, RootWindow(display, 0), RROutputChangeNotifyMask);
-    XSelectInput(display, RootWindow(display, 0), StructureNotifyMask);
-    XSync(display, False);
+    XRRSelectInput(*display, RootWindow(*display, 0), RROutputChangeNotifyMask);
+    XSelectInput(*display, RootWindow(*display, 0), StructureNotifyMask);
+    XSync(*display, False);
 }
 
 int main(int argc, char *argv[])
@@ -523,7 +537,7 @@ int main(int argc, char *argv[])
         }
     }
 
-    display = XOpenDisplay(NULL);
+    display = XOpenDisplay(getenv("DISPLAY"));
 
     if (display == NULL) {
         fprintf(stderr, "Unable to connect to X server\n");
@@ -542,7 +556,7 @@ int main(int argc, char *argv[])
         return EXIT_FAILURE;
     }
 
-    routine(display);
+    routine(&display);
 
     for (;;) {
         XEvent ev;
@@ -550,21 +564,20 @@ int main(int argc, char *argv[])
 
         do {
             XNextEvent(display, &ev);
-            usleep(100000);
             switch (ev.type - event_base) {
                 case RRNotify:
                     nev = (XRRNotifyEvent *) &ev;
                     if (nev->subtype == RRNotify_OutputChange) {
-                        routine(display);
+                        routine(&display);
                     }
                     break;
             }
             switch (ev.type) {
                 case ConfigureNotify:
-                    routine(display);
+                    routine(&display);
                     break;
             }
-        } while (XEventsQueued(display, QueuedAfterFlush));
+        } while (XPending(display));
     }
 
     XCloseDisplay(display);
